﻿using System;
using System.Collections.Generic;
//using System.Linq;
using System.Text;

namespace bnlstest
{
    public enum HeaderType { BNCS = 0xFF, W3GS = 0xF7, BNLS = 0x00};
    class Packet
    {
        

        private byte packetId;
        private byte[] data;
        private byte[] size;
        private int pos;
        private HeaderType header;

        public Packet(byte packetId, HeaderType header)
        {
            pos = 0;
            this.packetId = packetId;
            data = new byte[0];
            this.header = header;
        }

        public Packet(byte[] n_data)
        {
            // BNLS:
            // DWORD size
            // BYTE message ID
            // void data
            size = new byte[2];
            data = new byte[n_data.Length - 4];

            if (n_data[0] == (byte)HeaderType.BNCS || n_data[0] == (byte)HeaderType.W3GS)
            {
                byte FF = n_data[0];
                packetId = n_data[1];
                size[0] = n_data[2];
                size[1] = n_data[3];
                for (int count = 4; count < n_data.Length; ++count)
                    data[count - 4] = n_data[count];
            }
            else // bnls
            {
                size[0] = n_data[0];
                size[1] = n_data[1];
                packetId = n_data[2];
                for (int count = 0; count +4 < n_data.Length; count++)
                    data[count] = n_data[count + 3];
            }

        }

        public byte PID
        { get { return packetId; } }

        public byte[] Data
        {
            get
            {
                CalcSize();
                byte[] tmp = new byte[data.Length + 3];
                tmp[0] = size[0];
                tmp[1] = size[1];
                tmp[2] = packetId;
                for (int count = 0; count < data.Length; count++)
                    tmp[count + 3] = data[count];
                return tmp;
            }
        }
        public byte[] RawData
        {
            get
            {
                return data;
            }
            set
            {
                data = value;
            }
        }

        public void add(byte[] toAdd)
        {
            byte[] tmp = new byte[data.Length + toAdd.Length];
            Array.Copy(data, tmp, data.Length);
            for (int count = data.Length; count < tmp.Length; ++count)
                tmp[count] = toAdd[count - data.Length];
            data = tmp;
            CalcSize();
        }
        public void add(byte toAdd)
        {
            byte[] tmp = new byte[data.Length + 1];
            Array.Copy(data, tmp, data.Length);
            tmp[data.Length] = toAdd;
            data = tmp;
            CalcSize();
        }
        public void add(uint toAdd)
        {
            add(BitConverter.GetBytes(toAdd));
        }
        public void add(char[] toAdd)
        {
            ASCIIEncoding enc = new ASCIIEncoding();
            add(enc.GetBytes(toAdd));
        }
        public void add(char toAdd)
        {
            add(new ASCIIEncoding().GetBytes(new char[] { toAdd }));
        }
        public void add(string toAdd)
        {
            ASCIIEncoding enc = new ASCIIEncoding();
            add(enc.GetBytes(toAdd));
            add((byte)0x00); // null terminator!
        }
        public void add(string toAdd, System.Text.Encoding enc)
        {
            add(enc.GetBytes(toAdd));
            add((byte)0x00); // null terminator!
        }
        public void add(int toAdd)
        {
            add(BitConverter.GetBytes(toAdd));
        }
        public void add(bool toAdd)
        {
            add(BitConverter.GetBytes(toAdd));
        }
        public void add(short toAdd)
        {
            //add(BitConverter.GetBytes(toAdd));
            add(BitConverter.GetBytes(toAdd)[0]);
            add(BitConverter.GetBytes(toAdd)[1]);
        }

        public uint ReadUInt32()
        {
            uint i = BitConverter.ToUInt32(data, pos);
            pos += 4;
            return i;
        }
        public int ReadInt32()
        {
            int i = BitConverter.ToInt32(data, pos);
            pos += 4;
            return i;
        }
        public long ReadInt64()
        {
            long i = BitConverter.ToInt64(data, pos);
            pos += 8;
            return i;
        }
        public ulong ReadUInt64()
        {
            ulong i = BitConverter.ToUInt64(data, pos);
            pos += 8;
            return i;
        }
        public string ReadString()
        {
            string s = "";

            char[] next = new char[1];
            ASCIIEncoding enc = new ASCIIEncoding();

            next = enc.GetChars(data, pos++, 1);
            while (next[0] != (char)0 && pos < data.Length)
            {
                s += next[0];
                next = enc.GetChars(data, pos++, 1);
            }

            // returns how many characters are in the string
            return s;
        }
        public byte[] ReadByteArr(int count)
        {
            int bpos = pos;
            byte[] i = new byte[count];
            for (int c = pos; c < bpos + count; ++c, ++pos)
                i[c - bpos] = data[c];
            return i;
        }
        public short ReadShort()
        {
            short i = BitConverter.ToInt16(data, pos);
            pos += 2;
            return i;
        }
        public byte ReadByte()
        {
            return ReadByteArr(1)[0];
        }

        public override string ToString()
        {
            StringBuilder str = new StringBuilder(Data.Length);
            for (int count = 0; count < Data.Length; ++count)
            {
                str.Append(String.Format("{0:X2} ", Data[count]));
                if ((count + 1) % 8 == 0)
                    str.Append('\t');
                if ((count + 1) % 16 == 0)
                    str.Append('\n');
            }

            str.Append('\n');
            ASCIIEncoding enc = new ASCIIEncoding();
            for (int count = 0; count < Data.Length; ++count)
            {
                if (Data[count] >= (byte)0x20)
                {
                    str.Append(String.Format("{0:X2} ", enc.GetChars(new byte[1] { Data[count] })[0]));
                    if ((count + 1) % 8 == 0)
                        str.Append('\t');
                    if ((count + 1) % 16 == 0)
                        str.Append('\n');
                }
                else
                    str.Append(String.Format("{0:X2} ", "."));
                if ((count + 1) % 8 == 0)
                    str.Append('\t');
                if ((count + 1) % 16 == 0)
                    str.Append('\n');
            }

            return str.ToString();
        }
        public void CalcSize()
        {
            int intsize = data.Length;
                intsize += 3;
            size = BitConverter.GetBytes(intsize);
        }
    }
}